﻿/***************************************************************************************************
	© 2019 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 14370 $
$Date: 2019-10-25 15:55:37 -0500 (Fri, 25 Oct 2019) $
***************************************************************************************************/
-- Creates SQL objects used by Customer Service Alerts

set nocount on;

print 'Creating table dbo.tcsa_AlertDefinitions';
go

if object_id('tcsa_AlertDefinitions') is not NULL
begin;
	if (select rows from dbo.sysindexes where id = object_id('tcsa_AlertDefinitions') and indid < 2) > 0
	begin;
		exec sp_rename tcsa_AlertDefinitions, tcsa_AlertDefinitions_CSAInstallTemp;
	end;
	else
	begin;
		drop table dbo.tcsa_AlertDefinitions;
	end;
end;
go
if object_id('pkcsa_AlertDefinitions') is not NULL
begin;
	alter table dbo.tcsa_AlertDefinitions_CSAInstallTemp
		drop constraint pkcsa_AlertDefinitions;
end;
go
if object_id('tcsa_AlertDefinitions_CSAInstallTemp') is not NULL
begin;
	if not exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_AlertDefinitions_CSAInstallTemp' and name = 'RuleGroup')
	begin;
		alter table dbo.tcsa_AlertDefinitions_CSAInstallTemp
			add RuleGroup varchar(100) not NULL default '';
	end;

	if not exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_AlertDefinitions_CSAInstallTemp' and name = 'ALERT_ID')
	begin;
		alter table dbo.tcsa_AlertDefinitions_CSAInstallTemp
			add ALERT_ID varchar(20) not NULL default '';
	end;
end;
go

if object_id('tcsa_status_rules') is not NULL
begin;
	if (select rows from dbo.sysindexes where id = object_id('tcsa_status_rules') and indid < 2) > 0
	begin;
		exec sp_rename tcsa_status_rules, tcsa_status_rules_CSAInstallTemp;
	end;
	else
	begin;
		drop table dbo.tcsa_status_rules;
	end;
end;
go
if object_id('pkcsa_status_rules') is not NULL
begin;
	alter table dbo.tcsa_status_rules_CSAInstallTemp
		drop constraint pkcsa_status_rules;
end;
go
if object_id('tcsa_status_rules_CSAInstallTemp') is not NULL
begin;
	if not exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_status_rules_CSAInstallTemp' and name = 'RuleGroup')
	begin;
		alter table dbo.tcsa_status_rules_CSAInstallTemp
			add RuleGroup varchar(100) not NULL default '';
	end;

	if not exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_status_rules_CSAInstallTemp' and name = 'ALERT_ID')
	begin;
		alter table dbo.tcsa_status_rules_CSAInstallTemp
			add ALERT_ID varchar(20) not NULL default '';
	end;
end;
go

create table dbo.tcsa_AlertDefinitions
(
	SEQN int identity
		constraint pkcsa_AlertDefinitions primary key clustered,
	CRITERIA varchar(3072) not NULL default '',
	ENABLED bit not NULL default 1,
	DESCRIPTION varchar(255) not NULL default '',
	ALERT_TYPE varchar(10) not NULL default 'SQL',
	ALERT_ID varchar(20) not NULL default '',
	RuleGroup varchar(100) not NULL default ''
);
go
grant select, insert, update, delete on dbo.tcsa_AlertDefinitions to iMIS;
go

if object_id('tcsa_status_rules_CSAInstallTemp') is not NULL
begin;
	declare @oldrows int,
		@newrows int;

	insert dbo.tcsa_AlertDefinitions (CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup)
		select CRITERIA, ENABLED, '', 'SQL', ALERT_ID, RuleGroup
		from dbo.tcsa_status_rules_CSAInstallTemp
		order by RuleGroup, SEQN;

	set @newrows = @@rowcount;

	select @oldrows = count(*)
		from dbo.tcsa_status_rules_CSAInstallTemp;

	if @oldrows = @newrows
	begin;
		drop table dbo.tcsa_status_rules_CSAInstallTemp;
	end;
	else
	begin;
		print '!!! Error copying data from tcsa_status_rules_CSAInstallTemp into new table';
	end;
end;
go
if object_id('tcsa_AlertDefinitions_CSAInstallTemp') is not NULL
begin;
	declare @oldrows int,
		@newrows int;

	insert dbo.tcsa_AlertDefinitions (CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup)
		select CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup
		from dbo.tcsa_AlertDefinitions_CSAInstallTemp
		order by RuleGroup, SEQN;

	set @newrows = @@rowcount;

	select @oldrows = count(*)
		from dbo.tcsa_AlertDefinitions_CSAInstallTemp;

	if @oldrows = @newrows
	begin;
		drop table dbo.tcsa_AlertDefinitions_CSAInstallTemp;
	end;
	else
	begin;
		print '!!! Error copying data from tcsa_AlertDefinitions_CSAInstallTemp into new table';
	end;
end;
go
exec sp_recompile 'tcsa_AlertDefinitions';
go

print 'Creating table tcsa_AlertOutput (used by Xtender)';
go
if object_id('tcsa_Status_Output') is not NULL
begin;
	-- Always ok to delete
	drop table dbo.tcsa_Status_Output;
end;
go
if object_id('tcsa_AlertOutput') is not NULL
begin;
	-- Always ok to delete
	drop table dbo.tcsa_AlertOutput;
end;
go
create table dbo.tcsa_AlertOutput
(
	SEQN int identity
		constraint pkcsa_AlertOutput primary key clustered,
	OWNER_SPID smallint not NULL default @@spid,
	IMIS_USER varchar(50) not NULL default user_name(),
	EVENT_TYPE varchar(15) not NULL default 'ID',
	KEY_VALUE varchar(31) not NULL default '',
	RANK smallint not NULL default 0,						-- main window
	ICON smallint not NULL default 4,						-- detail
	KEYWORD varchar(40) not NULL default '',				-- detail
	DESCRIPTION varchar(4000) not NULL default '',			-- detail
	TOOLTIP_TEXT varchar(1000) not NULL default '',			-- detail
	TITLE_TEXT varchar(255) not NULL default '',			-- detail
	CLIPBOARD_TEXT text NOT NULL default '',				-- detail
	CLIPBOARD_DESCRIPTION varchar(100) not NULL default '',	-- detail
	ACTION_TEXT varchar(1000) not NULL default '',			-- detail
	ACTION_DESCRIPTION varchar(100) not NULL default '',	-- detail
	ACTION_TYPE varchar(30) not NULL default 'Launch',		-- detail
	LOG_TEXT varchar(1000) not NULL default '',				-- main window
	PLAY_SOUND bit not NULL default 0,						-- main window
	AUTO_POPUP bit not NULL default 0,						-- detail
	AUTO_CLIPBOARD bit not NULL default 0,					-- detail
	AUTO_ACTION bit not NULL default 0						-- detail
);
go
create index icsa_AlertOutputUSERRANK
	on dbo.tcsa_AlertOutput(OWNER_SPID, RANK);
go
grant select, insert, update, delete on dbo.tcsa_AlertOutput to iMIS;
go


print 'Creating table tcsa_AlertUsage (used by iPart)';
go
if object_id('tcsa_AlertUsage') is not NULL
begin;
	if (select rows from dbo.sysindexes where id = object_id('tcsa_AlertUsage') and indid < 2) > 0
	begin;
		exec sp_rename tcsa_AlertUsage, tcsa_AlertUsage_CSAInstallTemp;
	end;
	else
	begin;
		drop table dbo.tcsa_AlertUsage;
	end;
end;
go
if object_id('pkcsa_AlertUsage') is not NULL
begin;
	alter table dbo.tcsa_AlertUsage_CSAInstallTemp
		drop constraint pkcsa_AlertUsage;
end;
go
create table dbo.tcsa_AlertUsage
(
	SEQN int identity
		constraint pkcsa_AlertUsage primary key clustered,
	EventStart datetime not NULL,
	EventEnd datetime not NULL default getdate(),
	RuleGroup varchar(100) not NULL default '',
	RuleCount int not NULL default 0,
	RowsCount int not NULL default 0,
	ErrorCount int not NULL default 0,
	ActionCount int not NULL default 0,
	AutoActionCount int not NULL default 0,
	ID varchar(10) not NULL default '',
	UserId varchar(60) not NULL default '',
	Url varchar(1000) not NULL default ''
);
go
grant select, insert, update, delete on dbo.tcsa_AlertUsage to iMIS;
go
if object_id('tcsa_AlertUsage_CSAInstallTemp') is not NULL
begin
	insert dbo.tcsa_AlertUsage(EventStart, EventEnd, RuleGroup, RuleCount, RowsCount, ErrorCount,
			ActionCount, AutoActionCount, ID, UserId, Url)
		select EventStart, EventEnd, RuleGroup, RuleCount, RowsCount, ErrorCount,
			ActionCount, AutoActionCount, ID, UserId, Url
		from tcsa_AlertUsage_CSAInstallTemp
		order by SEQN;

	if @@rowcount = (select count(*) from dbo.tcsa_AlertUsage_CSAInstallTemp)
	begin
		drop table tcsa_AlertUsage_CSAInstallTemp
	end
	else
	begin
		print 'Error copying and dropping tcsa_AlertUsage_CSAInstallTemp'
	end
end
go

print 'Creating procedure pcsa_WriteSystemParams (used by Xtender)';
go
if object_id('pcsa_WriteSystemParams') is NULL
begin
	exec ('create /* model */ procedure pcsa_WriteSystemParams as set nocount on;');
end;
go
alter procedure pcsa_WriteSystemParams
	@sParameterName varchar(50),
	@sShortValue varchar(255)
as
/***************************************************************************************************
	© 2019 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 14370 $
$Date: 2019-10-25 15:55:37 -0500 (Fri, 25 Oct 2019) $
***************************************************************************************************/
begin;
	set nocount on;

	if @sParameterName is not NULL
	begin;
		if @sShortValue is NULL
		begin;
			delete dbo.System_Params
				where ParameterName = @sParameterName;
		end;
		else
		begin;
			insert dbo.System_Params (ParameterName, ShortValue)
				select @sParameterName, @sShortValue
				where not exists (select 1 from dbo.System_Params where ParameterName = @sParameterName);

			if @@rowcount = 0
			begin;
				update sp
					set ShortValue = @sShortValue
					from dbo.System_Params sp
					where sp.ParameterName = @sParameterName;
			end;
		end;
	end;
end;
go
grant execute on dbo.pcsa_WriteSystemParams to IMIS;
go



print 'Dropping obsolete procedure pcsa_calculate_status';
go
if object_id('pcsa_calculate_status') is not NULL
begin;
	drop procedure dbo.pcsa_calculate_status;
end;
go

print 'Adding security keyword for CSA Editor window (used by Xtender)';
go
if not exists (select 1 from dbo.Gen_Tables where TABLE_NAME = 'ACCESS_KEYWORDS'
		and upper(CODE) = 'CSAEDIT')
begin;
	insert dbo.Gen_Tables (TABLE_NAME, CODE, UPPER_CODE, DESCRIPTION)
		values ('ACCESS_KEYWORDS', 'CSAEdit', 'CSAEDIT', 'CSA Alert Editor');
end;
go

print 'Adding CSAEdit keyword to Manager, Administrator logins';
go
-- One row at a time to be as safe as possible with triggers
update dbo.Users
	set AccessKeywords = AccessKeywords + ',CSAEdit'
	where UserId = 'MANAGER'
		and upper(AccessKeywords) not like '%CSAEDIT%'
		and len(AccessKeywords) + 8 < 255;
update dbo.Users
	set AccessKeywords = AccessKeywords + ',CSAEdit'
	where UserId = 'ADMINISTRATOR'
		and upper(AccessKeywords) not like '%CSAEDIT%'
		and len(AccessKeywords) + 8 < 255;
go

print 'Creating table of TaskCentre servers, logins, passwords (used by Xtender)';
go
if not exists (select 1 from dbo.Gen_Tables where TABLE_NAME = 'CSA_TASKCENTRE_SERVERS')
begin;
	insert dbo.Gen_Tables (TABLE_NAME, CODE, UPPER_CODE, SUBSTITUTE, DESCRIPTION)
		values ('CSA_TASKCENTRE_SERVERS', 'ServerAlias', 'SERVERALIAS', 'RealServerName;aTCLoginForCSA;TCPassword',
			'TaskCentre server for use by CSA. Code = nickname referenced by alert. Expansion ='
			+ ' server;user;password to be used by CSA separated by semicolons.');
end;
go

print 'Creating configuration items (iPart version)';
go

-- Insert RSM settings group if not present
insert dbo.SystemConfigPageRef (SystemConfigPageKey, SystemConfigPageName, SystemConfigPageDesc)
	select '316F8E3A-8391-45E4-9005-BC5FDE08EFAC', 'RSMCustomSettings', 'Custom settings for RSM-created features'
	where not exists (select 1 from dbo.SystemConfigPageRef where SystemConfigPageKey = '316F8E3A-8391-45E4-9005-BC5FDE08EFAC');
go
if not exists (select 1 from dbo.SystemConfig where SystemConfigKey = '42FDD0CF-9892-4007-9FEC-B2E42FB0AFCF')
begin;
	print '-- adding RSM.CSA.License';

	exec dbo.asi_InsertSystemConfig 'Organization', 'RSMCustomSettings', 'MANAGER',
		'42FDD0CF-9892-4007-9FEC-B2E42FB0AFCF', 'RSM.CSA.License',
		'85F7051F-0D92-4802-A6CC-74E9B54EA825', 7, 7, 40,
		'<span class="HelperText">Customer Service Alerts settings</span><br/><br/>License key for Customer Service Alerts iParts',
		'', 1101;
end;
else
begin;
	print '-- RSM.CSA.License already exists';
	update dbo.SystemConfigPageParameterRef
		set SortOrder = 1101
		where ParameterName = 'RSM.CSA.License';
	update dbo.SystemConfigParameterRef
		set Description = '<span class="HelperText">Customer Service Alerts settings</span><br/><br/>License key for Customer Service Alerts iParts'
		where ParameterName = 'RSM.CSA.License';
end;
go
if not exists (select 1 from dbo.SystemConfig where SystemConfigKey = '2E23E97E-6D8A-4F3F-A59A-4C22E577849E')
begin;
	print '-- adding RSM.CSA.DefaultMarkupInOutput';

	exec dbo.asi_InsertSystemConfig 'Organization', 'RSMCustomSettings', 'MANAGER',
		'2E23E97E-6D8A-4F3F-A59A-4C22E577849E', 'RSM.CSA.DefaultMarkupInOutput',
		'CC72633A-56F5-4D48-B6EF-2006EE876540', 2, 0, 40,
		'Allow unescaped HTML in alert output by default. May be overridden by iPart configuration or alert output.',
		'False', 1102;
end;
else
begin;
	print '-- RSM.CSA.DefaultMarkupInOutput already exists';
	update dbo.SystemConfigPageParameterRef
		set SortOrder = 1102
		where ParameterName = 'RSM.CSA.DefaultMarkupInOutput'
end;
go
if not exists (select 1 from dbo.SystemConfig where SystemConfigKey = '72A671E1-CAFE-4234-8082-8C3CD203D38B')
begin;
	print '-- adding RSM.CSA.LogUsage';

	exec dbo.asi_InsertSystemConfig 'Organization', 'RSMCustomSettings', 'MANAGER',
		'72A671E1-CAFE-4234-8082-8C3CD203D38B', 'RSM.CSA.LogUsage',
		'EBC22BE5-C57D-4202-904D-A1D280C6EA35', 2, 0, 40,
		'Write statistics to tcsa_AlertUsage each time a RuleGroup is evaluated.',
		'True', 1103;
end;
else
begin;
	print '-- RSM.CSA.LogUsage already exists';
	update dbo.SystemConfigPageParameterRef
		set SortOrder = 1103
		where ParameterName = 'RSM.CSA.LogUsage'

end;
go

print 'Creating table tcsa_EventsForTaskCentre (used by iPart)';
go
if object_id('tcsa_EventsForTaskCentre') is not NULL
begin;
	if (select rows from dbo.sysindexes where id = object_id('tcsa_EventsForTaskCentre') and indid < 2) > 0
	begin;
		print '***Not deleting - data exists';
	end;
	else
	begin;
		drop table dbo.tcsa_EventsForTaskCentre;
	end;
end;
go
if object_id('tcsa_EventsForTaskCentre') is NULL
begin;
	create table dbo.tcsa_EventsForTaskCentre
	(
		-- Automatically assigned properties
		Seqn int identity,
		InsertedDate datetime not NULL default getdate(),
		InsertedBy nvarchar(128) not NULL default user_name(),
		OriginDatabase nvarchar(128) not NULL default db_name(),

		-- Populated by CSA
		ServerAlias varchar(60) not NULL,			-- Server alias, for compatibility with CSA desktop
		TaskId int not NULL,						-- Task which will handle this event
		UserName varchar(60) not NULL,				-- User who clicked the Action button
		SelectedId varchar(50) not NULL,			-- Contact to which the Action applies
		Param1 nvarchar(255) not NULL default '',	-- Optional parameter 1
		Param2 nvarchar(255) not NULL default '',	-- Optional parameter 2
		Choice int not NULL default -1,				-- Optional Radio Button Choice selected

		-- Defaulted during insert, task may update
		ProcessedDate datetime NULL,
		ProcessedStatus int not NULL default 0,
		ProcessedMessage nvarchar(255) not NULL default ''
	);
end;
go
if not exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_EventsForTaskCentre' and name = 'Choice')
begin;
	alter table dbo.tcsa_EventsForTaskCentre
		add Choice int not NULL default -1;
end;
go
if object_id('pkcsa_EventsForTaskCentre') is NULL
begin;
	alter table dbo.tcsa_EventsForTaskCentre
		add constraint pkcsa_EventsForTaskCentre
		primary key clustered (Seqn);
end;
go
if not exists (select 1 from dbo.sysindexes where id = object_id('tcsa_EventsForTaskCentre')
		and name = 'icsa_EventsForTaskCentreSelectedId')
begin;
	create index icsa_EventsForTaskCentreSelectedId
		on dbo.tcsa_EventsForTaskCentre(SelectedId);
end;
go
grant select, insert, update, delete, references, view definition on dbo.tcsa_EventsForTaskCentre to IMIS;
grant select on dbo.tcsa_EventsForTaskCentre to public;
go

print 'Creating table tcsa_ActionQueue (used by iPart)';
go
if object_id('tcsa_ActionQueue') is not NULL
begin;
	-- Always ok to delete
	drop table dbo.tcsa_ActionQueue;
end;
go
create table dbo.tcsa_ActionQueue
(
	PageInstanceKey uniqueidentifier not NULL,
	ActionItemKey uniqueidentifier not NULL,
	CreatedDate datetime not NULL default getdate(),
	ExecuteCount int not NULL default 0,
	LastExecuted datetime NULL,

	ActionType int not NULL default 0,
	LoggedInId varchar(10) not NULL default '',
	SubjectId varchar(10) not NULL default '',
	RuleSeqn int not NULL default 0,
	CommandText nvarchar(4000) not NULL default N'',
	Seqn int identity
		constraint pkcsa_ActionQueue primary key clustered
);
go
create unique index icsa_ActionQueueKeys
	on dbo.tcsa_ActionQueue(PageInstanceKey, ActionItemKey);
go
create index icsa_ActionQueueDates
	on dbo.tcsa_ActionQueue(CreatedDate, LastExecuted)
	include (Seqn);
go
grant select, insert, update, delete on dbo.tcsa_ActionQueue to iMIS;
go



print 'Creating procedure pcsa_Internal_SaveAction';
go
if object_id('pcsa_Internal_SaveAction') is NULL
begin
	exec ('create /* model */ procedure pcsa_Internal_SaveAction as set nocount on;');
end;
go
alter procedure pcsa_Internal_SaveAction
	@PageInstanceKey uniqueidentifier,
	@ActionItemKey uniqueidentifier,
	@ActionType int,
	@LoggedInId varchar(10),
	@SubjectId varchar(10),
	@RuleSeqn int,
	@CommandText nvarchar(4000)
as
/***************************************************************************************************
	© 2019 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 14370 $
$Date: 2019-10-25 15:55:37 -0500 (Fri, 25 Oct 2019) $
***************************************************************************************************/
-- Insert the provided action details to the action queue
begin;
	set nocount on;

	insert dbo.tcsa_ActionQueue (PageInstanceKey, ActionItemKey, ActionType, LoggedInId, SubjectId, RuleSeqn, CommandText)
		values (@PageInstanceKey, @ActionItemKey, @ActionType, @LoggedInId, @SubjectId, @RuleSeqn, @CommandText);
end;
go
grant execute, view definition on dbo.pcsa_Internal_SaveAction to IMIS;
go


print 'Creating procedure pcsa_Internal_GroomActions';
go
if object_id('pcsa_Internal_GroomActions') is NULL
begin
	exec ('create /* model */ procedure pcsa_Internal_GroomActions as set nocount on;');
end;
go
alter procedure pcsa_Internal_GroomActions
	@numDaysExecuted int = 60,
	@numDaysNotExecuted int = 7
as
/***************************************************************************************************
	© 2019 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 14370 $
$Date: 2019-10-25 15:55:37 -0500 (Fri, 25 Oct 2019) $
***************************************************************************************************/
-- Delete actions after a configured number of days. By default, 7 days for not-executed, 60 days for executed.
begin;
	set nocount on;

	declare @xCreatedCutoff datetime,
		@xExecutedCutoff datetime,
		@xOldRecordCutoff datetime;

	set @xCreatedCutoff = dateadd(day, -1 * abs(@numDaysNotExecuted), getdate());
	set @xExecutedCutoff = dateadd(day, -1 * abs(@numDaysExecuted), getdate());

	if @xCreatedCutoff < @xExecutedCutoff
	begin
		set @xOldRecordCutoff = @xExecutedCutoff;
	end
	else
	begin
		set @xOldRecordCutoff = @xCreatedCutoff;
	end

	delete dbo.tcsa_ActionQueue
		where CreatedDate < @xOldRecordCutoff
			and LastExecuted < @xExecutedCutoff
			and LastExecuted is not NULL;

	delete dbo.tcsa_ActionQueue
		where CreatedDate < @xOldRecordCutoff
			and CreatedDate < @xCreatedCutoff
			and LastExecuted is NULL;
end;
go
grant execute, view definition on dbo.pcsa_Internal_GroomActions to IMIS;
go


print 'Creating procedure pcsa_Internal_ExecuteAction';
go
if object_id('pcsa_Internal_ExecuteAction') is NULL
begin
	exec ('create /* model */ procedure pcsa_Internal_ExecuteAction as set nocount on;');
end;
go
alter procedure pcsa_Internal_ExecuteAction
	@PageInstanceKey uniqueidentifier,
	@ActionItemKey uniqueidentifier
as
/***************************************************************************************************
	© 2019 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 14370 $
$Date: 2019-10-25 15:55:37 -0500 (Fri, 25 Oct 2019) $
***************************************************************************************************/
-- Update the queue, return the action details to the caller to be executed
begin;
	set nocount on;

	declare @actionType int,
		@message nvarchar(255),
		@loggedInId varchar(10),
		@subjectId varchar(10),
		@commandText nvarchar(4000),
		@seqn int;

	set @message = '';

	update aq
		set ExecuteCount = ExecuteCount + 1,
			LastExecuted = getdate(),
			@actionType = ActionType,
			@loggedInId = LoggedInId,
			@subjectId = SubjectId,
			@commandText = CommandText,
			@seqn = Seqn
		from dbo.tcsa_ActionQueue aq
		where aq.PageInstanceKey = @pageInstanceKey
			and aq.ActionItemKey = @actionItemKey;

	if @@rowcount != 1
	begin;
		set @actionType = 0;
		set @message = 'No matching action found';
		goto done;
	end;

done:
	select @message 'ReturnMessage', @actionType 'ActionType', @loggedInId 'LoggedInId', @subjectId 'SubjectId',
		@commandText 'CommandText', @seqn 'Seqn';
end;
go
grant execute, view definition on dbo.pcsa_Internal_ExecuteAction to IMIS;
go

print 'Creating procedure pr_Internal_SaveUsage';
go
if object_id('pr_Internal_SaveUsage') is NULL
begin
	exec ('create /* model */ procedure pr_Internal_SaveUsage as set nocount on;');
end;
go
alter procedure pr_Internal_SaveUsage
	@eventStart datetime,
	@ruleGroup varchar(50),
	@ruleCount int,
	@rowsCount int,
	@errorCount int,
	@actionCount int,
	@autoActionCount int,
	@id varchar(10),
	@userId varchar(60),
	@url varchar(1000)
as
/***************************************************************************************************
	© 2019 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 14370 $
$Date: 2019-10-25 15:55:37 -0500 (Fri, 25 Oct 2019) $
***************************************************************************************************/
-- Log an event each time a rulegroup is executed
begin
	set nocount on;

	insert tcsa_AlertUsage (EventStart, EventEnd, RuleGroup, RuleCount, RowsCount, ErrorCount,
			ActionCount, AutoActionCount, ID, UserId, Url)
		values (@eventStart, getdate(), @ruleGroup, @ruleCount, @rowsCount, @errorCount,
			@actionCount, @autoActionCount, @id, @userId, @url)
end;
go
grant execute, view definition on dbo.pr_Internal_SaveUsage to IMIS;
go

print 'Creating procedure pr_csa_ExportAlertRules';
go
if object_id('pr_csa_ExportAlertRules') is NULL
begin
	exec ('create /* model */ procedure pr_csa_ExportAlertRules as set nocount on;');
end;
go
alter procedure pr_csa_ExportAlertRules
as
/***************************************************************************************************
	© 2019 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 14370 $
$Date: 2019-10-25 15:55:37 -0500 (Fri, 25 Oct 2019) $
***************************************************************************************************/
-- Export CSA alert definitions as a SQL script which can be run on another database
begin
	set nocount on;

	declare @criteria varchar(3072),
		@enabled bit,
		@description varchar(255),
		@alerttype varchar(10),
		@alertid varchar(20),
		@rulegroup varchar(100),

		@criteriasafe varchar(6144),
		@descriptionsafe varchar(510),
		@alerttypesafe varchar(20),
		@alertidsafe varchar(40),
		@rulegroupsafe varchar(200)


	declare @tScript table (SEQN int identity, SCRIPT varchar(8000) not NULL)

 	declare rule_cursor cursor local fast_forward
	for
	select CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup
		from dbo.tcsa_AlertDefinitions
		where 1 = 1
		order by RuleGroup, CRITERIA, SEQN;

	open rule_cursor;
	fetch next from rule_cursor
		into @criteria, @enabled, @description, @alerttype, @alertid, @rulegroup;

	while @@fetch_status <> -1
	begin
		if @@fetch_status = 0
		begin
			set @criteriasafe = replace(@criteria, '''', '''''');
			set @descriptionsafe = replace(@description, '''', '''''');
			set @alerttypesafe = replace(@alerttype, '''', '''''');
			set @alertidsafe = replace(@alertid, '''', '''''');
			set @rulegroupsafe = replace(@rulegroup, '''', '''''');

			insert @tScript (SCRIPT) values ('if not exists (select 1 from dbo.tcsa_AlertDefinitions where RuleGroup = ''' + @rulegroupsafe + '''')
			insert @tScript (SCRIPT) values ('	and CRITERIA = ''' + @criteriasafe + ''')')
			insert @tScript (SCRIPT) values ('begin')
			insert @tScript (SCRIPT) values ('	insert dbo.tcsa_AlertDefinitions (RuleGroup, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, CRITERIA)')
			insert @tScript (SCRIPT) values ('		values (''' + @rulegroupsafe + ''', ' + case @enabled when 1 then '1' else '0' end + ', '''
					+ @descriptionsafe + ''', ''' + @alerttypesafe + ''', ''' + @alertidsafe + ''',')
			insert @tScript (SCRIPT) values ('			''' + @criteriasafe + ''');')
			insert @tScript (SCRIPT) values ('end;')
			insert @tScript (SCRIPT) values ('')
		end;
		fetch next from rule_cursor
			into @criteria, @enabled, @description, @alerttype, @alertid, @rulegroup;
	end;
	close rule_cursor;
	deallocate rule_cursor;

	select SCRIPT
		from @tScript
		order by SEQN
end;
go
grant execute, view definition on dbo.pr_csa_ExportAlertRules to IMIS;
go

print 'Finished.';
go
